package edu.cmu.cs.fluid.Tallyho.analysis;

import java.util.*;

import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.dom.*;

import edu.cmu.cs.fluid.Tallyho.analysis.util.JavaEditorMarker;
import edu.cmu.cs.fluid.Tallyho.coe.*;

public final class ExtraImports {
  /**
   * Log4j logger for this class
   */
  private static final org.apache.log4j.Logger LOG =
    org.apache.log4j.Logger.getLogger("TALLYHO.ExtraImports");

  // Holds information about the current Java source file being examined
  private ICompilationUnit javaFile;
  private String javaOSFileName;
  private String javaFileSource;
  private CompilationUnit ast;

  private Map importUsed;
  private Map importSourceLine;

  private int importStatementCount = 0;
  private int unneededImportCount = 0;

  private class ImportUseVisitor extends ASTVisitor {
    public boolean visit(ImportDeclaration node) {
      return false; // skip these since we know them already
    }
    public boolean visit(QualifiedName node) {
      process(node);
      return true;
    }
    public boolean visit(SimpleName node) {
      process(node);
      return true;
    }
    void process(Name node) {
      ITypeBinding tb = node.resolveTypeBinding();
      if (tb != null) {
        String pn = packageName(tb);
        if (pn != null) {
          String fullPackageImport = pn + ".*";
          String classImport = pn + "." + typeName(tb);
          Set pkgs = importUsed.keySet();
          Iterator i = pkgs.iterator();
          while (i.hasNext()) {
            String importName = (String) i.next();
            if (importName.equals(fullPackageImport)
              || importName.equals(classImport)) {
              importUsed.put(importName, new Boolean(true));
            }
          }
        }
      }
    }
    String typeName(ITypeBinding t) {
      if (t.isArray()) {
        ITypeBinding et = t.getElementType();
        if (et != null) {
          return typeName(et);
        } else {
          System.out.println(
            "[ExtraImports] "
              + "ERROR array doesn't seem to have an element type in "
              + javaOSFileName
              + " : "
              + t);
          return "package"; // a keyword so it doesn't match our imports
        }
      } else {
        return t.getName();
      }
    }
    String packageName(ITypeBinding t) {
      if (t.isArray()) {
        ITypeBinding et = t.getElementType();
        if (et != null) {
          return packageName(et);
        } else {
          System.out.println(
            "[ExtraImports] "
              + "ERROR array doesn't seem to have an element type in "
              + javaOSFileName
              + " : "
              + t);
          return "package"; // a keyword so it doesn't match our imports
        }
      } else {
        IPackageBinding pb = t.getPackage();
        if (pb != null) {
          return pb.getName();
        } else {
          return null; // either primative or the null type
        }
      }
    }
  }

  private boolean allImportsUsed() {
    Collection c = importUsed.values();
    Iterator i = c.iterator();
    boolean result = true;
    while (i.hasNext()) {
      result = result && ((Boolean) i.next()).booleanValue();
    }
    return result;
  }

  private void reportRemainingUnused() {
    Set s = importUsed.keySet();
    Iterator i = s.iterator();
    while (i.hasNext()) {
      final String importName = (String) i.next();
      if (!((Boolean) importUsed.get(importName)).booleanValue()) {
        int lineNo = ((Integer) importSourceLine.get(importName)).intValue();
        new CoEResultsCollecter(
          "superflous package/class import(s)",
          "Superfluous import of " + importName,
          new JavaEditorMarker(lineNo, javaFile),
          importName,
          null);
        unneededImportCount++;
      }
    }
  }

  public boolean find(ICompilationUnit file, CompilationUnit ast)
    throws JavaModelException {
    this.ast = ast;
    javaFileSource = file.getSource();
    javaFile = file;
    javaOSFileName =
      javaFile.getResource().getLocation().makeAbsolute().toOSString();

    // get this file's list of imports
    IImportDeclaration[] imports = file.getImports();
    importUsed = new HashMap(imports.length);
    importSourceLine = new HashMap(imports.length);
    for (int i = 0; i < imports.length; i++) {
      // we will mark all imports as not being used
      importUsed.put(imports[i].getElementName(), new Boolean(false));

      // save the line of code where the import occurs
      int offset = imports[i].getSourceRange().getOffset();
      int lineNo = ast.lineNumber(offset);
      importSourceLine.put(imports[i].getElementName(), new Integer(lineNo));

      // recorce an occurance for each import in the file
      importStatementCount++;
    }

    ast.accept(new ImportUseVisitor()); // do the actual search

    reportRemainingUnused();
    ast = null; // avoid memory leak
    javaFile = null;
    return false; // everything went OK
  }

  public void recordResults() {
    LOG.info("-------------");
    LOG.info("EXTRA IMPORTS");
    LOG.info("-------------");
    LOG.info(" import statement count = " + importStatementCount);
    LOG.info(" unneeded import count = " + unneededImportCount);
    importStatementCount = 0;
    unneededImportCount = 0;
  }
}