package edu.cmu.cs.fluid.Tallyho.analysis.util;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.IPackageBinding;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.IVariableBinding;

public final class DOMUtilities {

  private DOMUtilities() {
  }

  public static String toFullName(IPackageBinding p) {
    return (p == null ? null : p.getName().intern());
  }

  public static String toFullName(IMethodBinding m) {
    return (
      m == null
        ? null
        : (m.getDeclaringClass() != null
          ? toFullName(m.getDeclaringClass()) + "." + m.getName()
          : m.getName())
          .intern());
  }

  public static String toFullName(ITypeBinding t) {
    if (t == null) {
      return null;
    }
    ITypeBinding baseType = (t.isArray() ? t.getElementType() : t);
    String result =
      (baseType.getPackage() != null
        ? baseType.getPackage().getName() + "."
        : "");
    ITypeBinding nt = t;
    while (nt.isNested()) {
      nt = nt.getDeclaringClass();
      result += nt.getName() + ".";
    }
    result += t.getName().intern();
    return result;
  }

  public static String toFullName(IVariableBinding v) {
    return (
      v == null
        ? null
        : (v.getDeclaringClass() != null
          ? toFullName(v.getDeclaringClass()) + "." + v.getName()
          : v.getName())
          .intern());
  }

  public static String toOSFileName(ICompilationUnit javaFile) {
    return (
      javaFile == null
        ? "null".intern()
        : javaFile
          .getResource()
          .getLocation()
          .makeAbsolute()
          .toOSString()
          .intern());
  }
}