package edu.cmu.cs.fluid.Tallyho.analysis.util;

import java.io.PrintStream;
import java.util.Iterator;
import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.ITypeBinding;

public final class SMethod {

  // key:String value:SMethod
  private static Map instanceMap = new WeakHashMap();

  /**
   * dumps instanceMap for debug purposes
   */
  public static void dump(PrintStream out) {
    if (out == null)
      out = System.out; // a reasonable default
    out.println("[SMethod] B E G I N   I N S T A N C E   D U M P");
    out.println("[SMethod] Current instance count = " + dumpInstanceCount());
    for (Iterator i = instanceMap.values().iterator(); i.hasNext();) {
      SMethod e = (SMethod) i.next();
      out.println("[SMethod] " + e.instanceKey);
    }
    out.println("[SMethod] E N D   I N S T A N C E   D U M P");
  }

  public static int dumpInstanceCount() {
    return instanceMap.size();
  }

  public static SMethod getInstance(IMethodBinding m) {
    if (m == null) {
      throw new NullPointerException();
    }
    SMethod newInstance = new SMethod();
    // build instance key
    newInstance.name = m.getName().intern();
    String returnType = DOMUtilities.toFullName(m.getReturnType());
    String declaringClassName = DOMUtilities.toFullName(m.getDeclaringClass());
    String frontPart = (returnType == null ? "" : returnType + " ");
    String rearPart = "(";
    for (int i = 0; i < m.getParameterTypes().length; i++) {
      String parameterType = DOMUtilities.toFullName(m.getParameterTypes()[i]);
      rearPart += (i == 0 ? parameterType : "," + parameterType);
    }
    rearPart += ")";
    newInstance.methodSignature =
      (frontPart + newInstance.name + rearPart).intern();
    newInstance.instanceKey =
      (frontPart + declaringClassName + "." + newInstance.name + rearPart)
        .intern();
    if (instanceMap.containsKey(newInstance.instanceKey)) {
      return (SMethod) instanceMap.get(newInstance.instanceKey);
    } else {
      instanceMap.put(newInstance.instanceKey, newInstance);
      newInstance.returnType = SType.getInstance(m.getReturnType());
      newInstance.parameterTypes = new SType[m.getParameterTypes().length];
      ITypeBinding declaringClass = m.getDeclaringClass();
      newInstance.declaringClass =
        (declaringClass == null ? null : SType.getInstance(declaringClass));
      for (int i = 0; i < m.getParameterTypes().length; i++) {
        newInstance.parameterTypes[i] =
          SType.getInstance(m.getParameterTypes()[i]);
      }
      // quick check for nulls
      if (newInstance.anyNull()) {
        System.out.println("[SMethod] NULL FIELDS DETECTED");
      }
      return newInstance;
    }
  }

  private SType declaringClass;
  private String instanceKey = "";
  private String methodSignature = "";
  private String name = "";
  private SType[] parameterTypes = new SType[0];
  private SType returnType;

  private SMethod() {
  }

  private boolean anyNull() {
    if (instanceKey == null
      || declaringClass == null
      || methodSignature == null
      || name == null
      || parameterTypes == null
      || returnType == null) {
      return true;
    }
    return false;
  }

  public SType getDeclaringClass() {
    return declaringClass;
  }

  public String getName() {
    return name;
  }

  public SType[] getParameterTypes() {
    return parameterTypes;
  }

  public SType getReturnType() {
    return returnType;
  }

  public boolean signatureMatches(SMethod m) {
    return (methodSignature == m.methodSignature ? true : false);
  }

  public String toString() {
    return instanceKey;
  }
}