package edu.cmu.cs.fluid.Tallyho.analysis.util;

import java.io.PrintStream;
import java.util.Iterator;
import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.IVariableBinding;

public final class SVariable {

  // key:String value:SVariable
  private static Map instanceMap = new WeakHashMap();

  /**
   * dumps instanceMap for debug purposes
   */
  public static void dump(PrintStream out) {
    if (out == null)
      out = System.out; // a reasonable default
    out.println("[SVariable] B E G I N   I N S T A N C E   D U M P");
    out.println("[SVariable] Current instance count = " + dumpInstanceCount());
    for (Iterator i = instanceMap.values().iterator(); i.hasNext();) {
      SVariable e = (SVariable) i.next();
      out.println("[SVariable] " + e.instanceKey);
    }
    out.println("[SVariable] E N D   I N S T A N C E   D U M P");
  }

  public static int dumpInstanceCount() {
    return instanceMap.size();
  }

  public static SVariable getInstance(IVariableBinding v) {
    if (v == null) {
      throw new NullPointerException();
    }
    SVariable newInstance = new SVariable();
    // build instance key
    newInstance.name = v.getName().intern();
    newInstance.isField = v.isField();
    newInstance.variableBindingAddress = v.toString();
    String declaringClassName = DOMUtilities.toFullName(v.getDeclaringClass());
    String type = DOMUtilities.toFullName(v.getType());
    newInstance.instanceKey =
      (newInstance.isField
        ? declaringClassName + "." + newInstance.name + ":" + type
        : "local:" + newInstance.name + newInstance.variableBindingAddress);
    if (instanceMap.containsKey(newInstance.instanceKey)) {
      return (SVariable) instanceMap.get(newInstance.instanceKey);
    } else {
      instanceMap.put(newInstance.instanceKey, newInstance);
      ITypeBinding declaringClass = v.getDeclaringClass();
      newInstance.declaringClass =
        (declaringClass == null ? null : SType.getInstance(declaringClass));
      newInstance.type = SType.getInstance(v.getType());
      // quick check for nulls
      if (newInstance.anyNull()) {
        System.out.println("[SVariable] NULL FIELDS DETECTED");
      }
      return newInstance;
    }
  }

  private SType declaringClass;
  private String instanceKey ="";
  private boolean isField;
  private String name = "";
  private SType type;
  private String variableBindingAddress;

  private SVariable() {
  }

  private boolean anyNull() {
    if (instanceKey == null
      || name == null
      || type == null
      || variableBindingAddress == null) {
      return true;
    }
    if (isField && declaringClass == null) {
      // might be the screwy ".length" that arrays have built-in
      if (name != "length") {
        return true;
      }
    }
    return false;
  }

  public SType getDeclaringClass() {
    return declaringClass;
  }

  public String getName() {
    return name;
  }

  public SType getType() {
    return type;
  }

  public boolean isField() {
    return isField;
  }

  public String toString() {
    return name + ":" + type;
  }
}