package edu.cmu.cs.fluid.Tallyho.coe;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import edu.cmu.cs.fluid.Tallyho.analysis.util.JavaEditorMarker;

public class CoELink implements IAdaptable {

  private static final Image defaultImage =
    ImageDescriptor.getMissingImageDescriptor().createImage();

  private static CoELink root = new CoELink("default");

  public static CoELink getRoot() {
    return root;
  }
  public static void resetRoot() {
    root = new CoELink("default");
  }
  public static void setRoot(CoELink rootNode) {
    root = rootNode;
  }

  private CoELink anchor; // link to parent instance
  private int counter = 0;
  private String description;
  private Image image;
  private List links = new LinkedList();
  private JavaEditorMarker location = null;
  private CoELinkFormatter output = null;
  private String sort;

  public CoELink(String description) {
    this.description = description;
    image = defaultImage;
    sort = description; // a default sort string
  }
  public void addLink(CoELink link) {
    links.add(link);
    link.setAnchor(this);
  }
  public Object getAdapter(Class key) {
    return null;
  }
  public CoELink getAnchor() {
    return anchor;
  }
  public int getCounter() {
    return counter;
  }
  public String getDescription() {
    return description;
  }
  public IMarker getEditorPosition() {
    if (location == null)
      return null;
    return location.getLocation();
  }
  public Image getImage() {
    return image;
  }
  public ICompilationUnit getJavaFile() {
    if (location == null)
      return null;
    return location.getJavaFile();
  }
  public int getLineNo() {
    if (location == null)
      return 1;
    return location.getLineNumber();
  }
  public int getLinkCount() {
    return links.size();
  }
  public CoELink[] getLinks() {
    return (CoELink[]) links.toArray(new CoELink[links.size()]);
  }
  public String getSortString() {
    return sort;
  }
  public boolean hasLinks() {
    return links.size() > 0;
  }
  public void incrementCounter() {
    counter++;
  }
  public void removeLink(CoELink link) {
    links.remove(link);
    link.setAnchor(null);
  }
  private void setAnchor(CoELink anchor) {
    this.anchor = anchor;
  }
  public void setCounter(int value) {
    counter = value;
  }
  public void setDescription(String description) {
    this.description = description;
  }
  public void setFormatter(CoELinkFormatter output) {
    this.output = output;
  }
  public void setImage(Image image) {
    this.image = image;
  }
  public void setJavaEditorLocation(JavaEditorMarker location) {
    this.location = location;
  }
  public void setSortString(String value) {
    sort = value;
  }
  public void sortLinks() {
    Comparator comparator = new CoELinkComparator();
    Collections.sort(links, comparator);
  }
  public String toString() {
    if (output != null) {
      return output.format(this);
    } else {
      return getDescription();
    }
  }
}