package edu.cmu.cs.fluid.Tallyho.coe;

import org.eclipse.jdt.core.IPackageDeclaration;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.swt.graphics.Image;

import edu.cmu.cs.fluid.Tallyho.TallyhoPlugin;
import edu.cmu.cs.fluid.Tallyho.analysis.util.JavaEditorMarker;

public final class CoEResultsCollecter {

  private static CoELink byCategory = new CoELink("byCategory");
  private static CoELink byCatAlt = new CoELink("byCatAlt");
  private static CoELink byProject = new CoELink("byProject");
  private static CoELinkFormatter format = new CoELinkFormatter() {
    public String format(CoELink link) {
      return link.getDescription()
        + " ("
        + link.getCounter()
        + (link.getCounter() > 1 ? " issues)" : " issue)");
    }
  };
  private static CoELinkFormatter catFormat = new CoELinkFormatter() {
    public String format(CoELink link) {
      return link.getCounter() + " " + link.getDescription();
    }
  };

  private static int issueCount = 0;

  final public static Image ALT_LOC_IMG =
    TallyhoPlugin.getImageDescriptor("altview.gif").createImage();
  final public static Image CHAIN_LINK_IMG =
    TallyhoPlugin.getImageDescriptor("link.gif").createImage();
  final public static Image JAVA_COMP_UNIT_IMG =
    TallyhoPlugin.getImageDescriptor("jcu.gif").createImage();
  final public static Image JAVA_PKG_IMG =
    TallyhoPlugin.getImageDescriptor("package.gif").createImage();
  final public static Image JAVA_PROJ_IMG =
    TallyhoPlugin.getImageDescriptor("proj.gif").createImage();
  final public static Image BROKEN_LINK_IMG =
    TallyhoPlugin.getImageDescriptor("brokenlink.gif").createImage();

  private static CoELink newProblemLink(CoEResultsCollecter r) {
    // create common problem report
    CoELink result = new CoELink(r.description);
    if (r.supportingInformation != null) {
      CoELink[] items = r.supportingInformation.getLinks();
      for (int i = 0; i < items.length; i++) {
        result.addLink(items[i]);
      }
    }
    result.setSortString(r.location.getPaddedLineNumber());
    result.setJavaEditorLocation(r.location);
    result.setImage(BROKEN_LINK_IMG);
    result.setFormatter(new CoELinkFormatter() {
      public String format(CoELink link) {
        return "line " + link.getLineNo() + ": " + link.getDescription();
      }
    });
    return result;
  }

  private static void addToCoETrees(CoEResultsCollecter r) {
    CoELink cat, lCat, alt, lAlt, prj, lPrj, pkg, lPkg, fil, lFil;
    CoELinkFormatter format = new CoELinkFormatter() {
      public String format(CoELink link) {
        return link.getDescription()
          + " ("
          + link.getCounter()
          + (link.getCounter() > 1 ? " issues)" : " issue)");
      }
    };
    
    CoELink problem = newProblemLink(r);
    
    // create a standalone "by category" tree
    cat = new CoELink(r.category);
    cat.setImage(BROKEN_LINK_IMG);
    cat.setFormatter(catFormat);
    prj = new CoELink(r.project);
    prj.setImage(JAVA_PROJ_IMG);
    prj.setFormatter(format);
    pkg = new CoELink(r.javaPackage);
    pkg.setImage(JAVA_PKG_IMG);
    pkg.setFormatter(format);
    fil = new CoELink(r.javaFileName);
    fil.setImage(JAVA_COMP_UNIT_IMG);
    fil.setFormatter(format);
    // add into existing "by category" tree
    lCat = findMatch(byCategory.getLinks(), r.category);
    if (lCat == null) {
      byCategory.addLink(cat);
      lCat = cat;
    }
    lPrj = findMatch(lCat.getLinks(), r.project);
    if (lPrj == null) {
      lCat.addLink(prj);
      lPrj = prj;
    }
    lPkg = findMatch(lPrj.getLinks(), r.javaPackage);
    if (lPkg == null) {
      lPrj.addLink(pkg);
      lPkg = pkg;
    }
    lFil = findMatch(lPkg.getLinks(), r.javaFileName);
    if (lFil == null) {
      lPkg.addLink(fil);
      lFil = fil;
    }
    lFil.addLink(problem);
    lFil.sortLinks();
    lFil.incrementCounter();
    lPkg.sortLinks();
    lPkg.incrementCounter();
    lPrj.sortLinks();
    lPrj.incrementCounter();
    lCat.sortLinks();
    lCat.incrementCounter();
    byCategory.sortLinks();

    // create a standalone "by category alternate" tree
    cat = new CoELink(r.category);
    cat.setImage(BROKEN_LINK_IMG);
    cat.setFormatter(catFormat);
    alt = new CoELink(r.alternateLocation);
    alt.setImage(ALT_LOC_IMG);
    alt.setFormatter(format);
    prj = new CoELink(r.project);
    prj.setImage(JAVA_PROJ_IMG);
    prj.setFormatter(format);
    pkg = new CoELink(r.javaPackage);
    pkg.setImage(JAVA_PKG_IMG);
    pkg.setFormatter(format);
    fil = new CoELink(r.javaFileName);
    fil.setImage(JAVA_COMP_UNIT_IMG);
    fil.setFormatter(format);
    // add into existing "by category alternate" tree
    lCat = findMatch(byCatAlt.getLinks(), r.category);
    if (lCat == null) {
      byCatAlt.addLink(cat);
      lCat = cat;
    }
    lAlt = findMatch(lCat.getLinks(), r.alternateLocation);
    if (lAlt == null) {
      lCat.addLink(alt);
      lAlt = alt;
    }
    lPrj = findMatch(lAlt.getLinks(), r.project);
    if (lPrj == null) {
      lAlt.addLink(prj);
      lPrj = prj;
    }
    lPkg = findMatch(lPrj.getLinks(), r.javaPackage);
    if (lPkg == null) {
      lPrj.addLink(pkg);
      lPkg = pkg;
    }
    lFil = findMatch(lPkg.getLinks(), r.javaFileName);
    if (lFil == null) {
      lPkg.addLink(fil);
      lFil = fil;
    }
    lFil.addLink(problem);
    lFil.sortLinks();
    lFil.incrementCounter();
    lPkg.sortLinks();
    lPkg.incrementCounter();
    lPrj.sortLinks();
    lPrj.incrementCounter();
    lAlt.sortLinks();
    lAlt.incrementCounter();
    lCat.sortLinks();
    lCat.incrementCounter();
    byCatAlt.sortLinks();

    // create a standalone "by project" tree
    prj = new CoELink(r.project);
    prj.setImage(JAVA_PROJ_IMG);
    prj.setFormatter(format);
    pkg = new CoELink(r.javaPackage);
    pkg.setImage(JAVA_PKG_IMG);
    pkg.setFormatter(format);
    fil = new CoELink(r.javaFileName);
    fil.setImage(JAVA_COMP_UNIT_IMG);
    fil.setFormatter(format);
    // add into existing "by project" tree
    lPrj = findMatch(byProject.getLinks(), r.project);
    if (lPrj == null) {
      byProject.addLink(prj);
      lPrj = prj;
    }
    lPkg = findMatch(lPrj.getLinks(), r.javaPackage);
    if (lPkg == null) {
      lPrj.addLink(pkg);
      lPkg = pkg;
    }
    lFil = findMatch(lPkg.getLinks(), r.javaFileName);
    if (lFil == null) {
      lPkg.addLink(fil);
      lFil = fil;
    }
    lFil.addLink(problem);
    lFil.sortLinks();
    lFil.incrementCounter();
    lPkg.sortLinks();
    lPkg.incrementCounter();
    lPrj.sortLinks();
    lPrj.incrementCounter();
    byProject.sortLinks();
  }

  private static CoELink findMatch(CoELink[] list, String description) {
    for (int i = 0; i < list.length; i++) {
      if (list[i].getDescription() == description) {
        return list[i];
      }
    }
    return null; // didn't find "description" in the given list
  }

  public static CoELink getByCategory() {
    return byCategory;
  }

  public static CoELink getByCategoryAlternative() {
    return byCatAlt;
  }

  public static CoELink getByProject() {
    return byProject;
  }

  public static int getIssueCount() {
    return issueCount;
  }

  public static void reset() {
    issueCount = 0;
    byCategory = new CoELink("byCategory");
    byCatAlt = new CoELink("byCatAlt");
    byProject = new CoELink("byProject");
  }

  private String category;
  private String description;
  private String javaFileName;
  private String javaPackage;
  private JavaEditorMarker location;
  private String alternateLocation;
  private String project;
  private CoELink supportingInformation;

  public CoEResultsCollecter(
    String category,
    String description,
    JavaEditorMarker location,
    String alternateLocation,
    CoELink supportingInformation) {
    this.category = category.intern();
    this.description = description.intern();
    javaFileName = location.getJavaFile().getElementName().intern();
    project = location.getJavaFile().getJavaProject().getElementName().intern();
    IPackageDeclaration[] pkgs;
    try {
      pkgs = location.getJavaFile().getPackageDeclarations();
      if (pkgs.length == 0) {
        javaPackage = "(default)".intern();
      } else {
        javaPackage = pkgs[0].getElementName().intern();
      }
    } catch (JavaModelException e) {
      //@ ignore
    } finally {
      if (javaPackage == null) {
        javaPackage = "(unknown)".intern();
      }
    }
    this.location = location;
    this.alternateLocation = alternateLocation.intern();
    this.supportingInformation = supportingInformation;

    issueCount++;
    addToCoETrees(this);
  }

  public String toString() {
    return "["
      + category
      + " : "
      + description
      + " : "
      + project
      + " : "
      + javaPackage
      + " : "
      + javaFileName
      + " : at line "
      + location.getLineNumber()
      + "]";
  }
}