package edu.cmu.cs.fluid.Tallyho.views;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.action.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.*;
import org.eclipse.ui.part.*;

import edu.cmu.cs.fluid.Tallyho.TallyhoPlugin;
import edu.cmu.cs.fluid.Tallyho.coe.*;
import edu.cmu.cs.fluid.Tallyho.analysis.*;

public class TallyhoView extends ViewPart {
  private TreeViewer viewer;
  private DrillDownAdapter drillDownAdapter;
  private Action actionPerformAnalysis;
  private Action actionOrderResults;
  private Action actionByCategory;
  private Action actionByCategoryAlternative;
  private Action actionByProject;
  private Action actionClear;
  private Action doubleClickAction;
  private final String byCategoryLabel = "Order Results by Category";
  private final String byCategoryAlternativeLabel =
    "Order Results by Category and Sub-Category";
  private final String byProjectLabel = "Order Results by Project";
  boolean byCategory = true;
  boolean byCategoryAlternative = false;

  class ViewContentProvider
    implements IStructuredContentProvider, ITreeContentProvider {
    private CoELink invisibleRoot;
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
    }
    public void dispose() {
    }
    public Object[] getElements(Object parent) {
      if (byCategory) {
        CoELink.setRoot(CoEResultsCollecter.getByCategory());
      } else if (byCategoryAlternative) {
        CoELink.setRoot(CoEResultsCollecter.getByCategoryAlternative());
      } else {
        CoELink.setRoot(CoEResultsCollecter.getByProject());
      }
      CoELink root = CoELink.getRoot();
      if (root.hasLinks()) {
        return getChildren(root);
      } else {
        return new Object[] {
        };
      }
    }
    public Object getParent(Object child) {
      if (child instanceof CoELink) {
        return ((CoELink) child).getAnchor();
      }
      return null;
    }
    public Object[] getChildren(Object parent) {
      return ((CoELink) parent).getLinks();
    }
    public boolean hasChildren(Object parent) {
      return ((CoELink) parent).hasLinks();
    }
  }

  class ViewLabelProvider extends LabelProvider {
    public String getText(Object obj) {
      return obj.toString();
    }
    public Image getImage(Object obj) {
      return ((CoELink) obj).getImage();
    }
  }

  class NameSorter extends ViewerSorter {
  }

  /**
   * The constructor.
   */
  public TallyhoView() {
  }

  /**
   * This is a callback that will allow us
   * to create the viewer and initialize it.
   */
  public void createPartControl(Composite parent) {
    viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
    //drillDownAdapter = new DrillDownAdapter(viewer);
    viewer.setContentProvider(new ViewContentProvider());
    viewer.setLabelProvider(new ViewLabelProvider());
    //viewer.setSorter(new NameSorter());
    viewer.setInput(ResourcesPlugin.getWorkspace());
    makeActions();
    hookContextMenu();
    hookDoubleClickAction();
    contributeToActionBars();
  }

  private void hookContextMenu() {
    MenuManager menuMgr = new MenuManager("#PopupMenu");
    menuMgr.setRemoveAllWhenShown(true);
    menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager) {
        TallyhoView.this.fillContextMenu(manager);
      }
    });
    Menu menu = menuMgr.createContextMenu(viewer.getControl());
    viewer.getControl().setMenu(menu);
    getSite().registerContextMenu(menuMgr, viewer);
  }

  private void contributeToActionBars() {
    IActionBars bars = getViewSite().getActionBars();
    fillLocalPullDown(bars.getMenuManager());
    fillLocalToolBar(bars.getToolBarManager());
  }

  private void fillLocalPullDown(IMenuManager manager) {
    manager.add(actionPerformAnalysis);
    manager.add(new Separator());
    manager.add(actionByCategory);
    manager.add(actionByCategoryAlternative);
    manager.add(actionByProject);
    manager.add(new Separator());
    manager.add(actionClear);
  }

  private void fillContextMenu(IMenuManager manager) {
    manager.add(actionPerformAnalysis);
    manager.add(new Separator());
    manager.add(actionByCategory);
    manager.add(actionByCategoryAlternative);
    manager.add(actionByProject);
    manager.add(new Separator());
    manager.add(actionClear);
    //manager.add(new Separator());
    //drillDownAdapter.addNavigationActions(manager);
    // Other plug-ins can contribute there actions here
    manager.add(new Separator("Additions"));
  }

  private void fillLocalToolBar(IToolBarManager manager) {
    manager.add(actionPerformAnalysis);
    manager.add(actionOrderResults);
    //manager.add(new Separator());
    //drillDownAdapter.addNavigationActions(manager);
  }

  private void makeActions() {
    // perform analysis action (search for quality problems)
    actionPerformAnalysis = new Action() {
      public void run() {
        doAnalysis();
      }
    };
    String actionText = "Search for Project Quality Issues";
    actionPerformAnalysis.setText(actionText);
    actionPerformAnalysis.setToolTipText(actionText);
    actionPerformAnalysis.setImageDescriptor(
      TallyhoPlugin.getImageDescriptor("search.gif"));

    // toggle between different results orders
    actionOrderResults = new Action() {
      public void run() {
        if (byCategory) {
          // toggle to category alternative view
          actionByCategoryAlternative.run();
        } else if (byCategoryAlternative) {
          // toggle to project view
          actionByProject.run();
        } else {
          // toggle to category view
          actionByCategory.run();
        }
      }
    };
    actionOrderResults.setImageDescriptor(
      TallyhoPlugin.getImageDescriptor("order.gif"));

    // set to by category view
    actionByCategory = new Action() {
      public void run() {
        byCategory = true;
        byCategoryAlternative = false;
        setViewState();
        viewer.refresh();
      }
    };
    actionByCategory.setText(byCategoryLabel);
    actionByCategory.setToolTipText(byCategoryLabel);
    actionByCategory.setImageDescriptor(
      TallyhoPlugin.getImageDescriptor("order.gif"));

    // set to by category alternative view
    actionByCategoryAlternative = new Action() {
      public void run() {
        byCategory = false;
        byCategoryAlternative = true;
        setViewState();
        viewer.refresh();
      }
    };
    actionByCategoryAlternative.setText(byCategoryAlternativeLabel);
    actionByCategoryAlternative.setToolTipText(byCategoryAlternativeLabel);
    actionByCategoryAlternative.setImageDescriptor(
      TallyhoPlugin.getImageDescriptor("order.gif"));

    // set to by project view
    actionByProject = new Action() {
      public void run() {
        byCategory = false;
        byCategoryAlternative = false;
        setViewState();
        viewer.refresh();
      }
    };
    actionByProject.setText(byProjectLabel);
    actionByProject.setToolTipText(byProjectLabel);
    actionByProject.setImageDescriptor(
      TallyhoPlugin.getImageDescriptor("order.gif"));

    // clear results action
    actionClear = new Action() {
      public void run() {
        CoEResultsCollecter.reset();
        viewer.refresh();
        setTitle("Code Quality Advice");
      }
    };
    actionText = "Clear Results";
    actionClear.setText(actionText);
    actionClear.setToolTipText(actionText);
    actionClear.setImageDescriptor(
      TallyhoPlugin.getImageDescriptor("clear_co.gif"));

    // detect double-click on items in our view
    doubleClickAction = new Action() {
      public void run() {
        ISelection selection = viewer.getSelection();
        Object obj = ((IStructuredSelection) selection).getFirstElement();
        CoELink link = (CoELink) obj;
        if (link.getJavaFile() != null) {
          try {
            IEditorPart ep = JavaUI.openInEditor(link.getJavaFile());
            if (link.getEditorPosition() != null) {
              ep.gotoMarker(link.getEditorPosition());
            }
          } catch (PartInitException e) {
            showMessage("PartInitException was thrown");
          } catch (org.eclipse.core.runtime.CoreException e) {
            showMessage("CoreException was thrown");
          }
        }
      }
    };
    setViewState();
  }

  private void hookDoubleClickAction() {
    viewer.addDoubleClickListener(new IDoubleClickListener() {
      public void doubleClick(DoubleClickEvent event) {
        doubleClickAction.run();
      }
    });
  }

  private void showMessage(String message) {
    MessageDialog.openInformation(
      viewer.getControl().getShell(),
      "Quality Advice",
      message);
  }

  /**
   * Passing the focus request to the viewer's control.
   */
  public void setFocus() {
    viewer.getControl().setFocus();
  }

  private void setViewState() {
    if (byCategory) {
      // category view (next is category alternative view)
      actionOrderResults.setText(byCategoryAlternativeLabel);
      actionOrderResults.setToolTipText(byCategoryAlternativeLabel);
    } else if (byCategoryAlternative) {
      // category alternative view (next is project view)
      actionOrderResults.setText(byProjectLabel);
      actionOrderResults.setToolTipText(byProjectLabel);
    } else {
      // project view (next is category view)
      actionOrderResults.setText(byCategoryLabel);
      actionOrderResults.setToolTipText(byCategoryLabel);
    }
    // set menu state
    actionByCategory.setChecked(byCategory);
    actionByCategory.setEnabled(!byCategory);
    actionByCategoryAlternative.setChecked(byCategoryAlternative);
    actionByCategoryAlternative.setEnabled(!byCategoryAlternative);
    actionByProject.setChecked(!byCategory && !byCategoryAlternative);
    actionByProject.setEnabled(byCategory || byCategoryAlternative);
  }

  /**
   * Analyse the Java code within the project for quality issues
   */
  private void doAnalysis() {
    ProgressMonitorDialog progressDialog =
      new ProgressMonitorDialog(viewer.getControl().getShell());
    try {
      IRunnableWithProgress op = new Majordomo();
      progressDialog.run(true, true, op);
    } catch (InvocationTargetException e) {
      showMessage(
        "TallyhoView.doAnalysis:InvocationTargetException"
          + "...probably a thread/code bug in Majordomo");
      actionClear.run(); // clear out possible junk results
      return;
    } catch (InterruptedException e) {
      if (!e.getMessage().equals("user cancel")) {
        showMessage("Analysis failed..." + e.getMessage());
        actionClear.run();
        return;
      }
    }
    viewer.refresh(); // must be here (maybe threading issue)
    int issueCount = CoEResultsCollecter.getIssueCount();
    setTitle(
      "Code Quality Advice"
        + (issueCount > 0
          ? " (" + issueCount + " issue" + (issueCount > 1 ? "s)" : ")")
          : ""));
  }
}